#include <string.h>
#include <malloc.h>

#include <iostream>
#include <fstream>

char *sp504header;
char *cardname;
char *device;
char *buffer;

size_t offset = 65536;

struct filelist
{
  char name[0x17];
  size_t size;
  size_t start;
  std::ifstream filehandle;
};


int main(int argc, char **argv)
{

  if (argc < 4)
    {
      std::cout << std::endl;
      std::cout << "    sp504write version 0.1" << std::endl;
      std::cout << "    (c) 2003 by Stefan Heimers" << std::endl;
      std::cout << "" << std::endl; 
      std::cout << "usage:" << std::endl; 
      std::cout << "sp504write <cardname> <device> <file1.mp3> [file2.mp3] ... [filen.mp3]" << std::endl; 
      exit(0);
    }

  cardname = argv[1]; 
  device = argv[2];

  struct filelist files[argc - 3];

  std::ofstream card(device,std::ios::out);
  if ( !card.is_open() )
    {
      std::cerr << "could not open card reader" << std::endl;
      exit(1);
    }

 

  // implementation of the M3C file format  PEROS Media GmbH 
  // implemented by stefan heimers <stefan@heimers.ch>

  sp504header = (char *)malloc(0x35);

  // card type
  sp504header[0x0] = 0x01;
  sp504header[0x1] = 0x00;


  // header version
  sp504header[0x2] = 0x01;

  // not used
  sp504header[0x3] = 0x00;

  // number of pieces of music (2 bytes)
  sp504header[0x4] = argc-3; // low byte
  sp504header[0x5] = (argc-3)/256; // high byte

  // start adress of playlist (4 bytes): 512
  sp504header[0x6] = 0x00;
  sp504header[0x7] = 0x02;
  sp504header[0x8] = 0x00;
  sp504header[0x9] = 0x00;
  // unused (4 bytes)
  sp504header[0xa] = 0;


  // name of card (36 bytes)
  strncpy(sp504header + 0xe,cardname,0x24);


  // Length of play list entry  (2 bytes)
  sp504header[0x33] = 0x20; // low byte
  sp504header[0x34] = 0;    // high byte 


  card.write(sp504header,0x35);




  // go to the start of the playlist

  card.seekp(512);

  // first, create a list of filenames and sizes
  for (int i=3; i<argc; i++)
	{
	  
      files[i-3].start = offset; // address on flash for start of mp3 data
	  
      files[i-3].filehandle.open(argv[i],std::ios::in);
      
      if(! (files[i-3].filehandle) )  // test, if the file really was opened
		std::cerr << "cannot open " << argv[i] << std::endl;
	  
      files[i-3].filehandle.seekg (0, std::ios::end);
      files[i-3].size = files[i-3].filehandle.tellg();
      files[i-3].filehandle.seekg (0, std::ios::beg);
	  
      offset += files[i-3].size;
	  
      for (int k=0;k<0x16;k++)
		files[i-3].name[k]=' ';
      files[i-3].name[0x17]='\0';
      if (strlen(argv[i]) >= 0x16)
		strncpy(files[i-3].name,argv[i],0x16 );
      else
		strncpy(files[i-3].name,argv[i],strlen(argv[i]) );
	  
	  
#ifdef DEBUG
      std::cerr << i << " size: " <<  files[i-3].size  << " name: " << files[i-3].name << " start: " << files[i-3].start << std::endl;
#endif
	  
      // now write one playlist entry
      card.write( (char *) &( files[i-3].start),4);
      card.write( (char *) &( files[i-3].size),4);
      card.write(files[i-3].name,0x17);
      //card.put( (char) 1);
	  card.put( (char) 1);
    }

  buffer = (char *) malloc(512);
  for (int i=3; i<argc; i++)
    {
      std::cout << "writing " << argv[i] <<  " to the memory card"  << std::endl;


      card.seekp(files[i-3].start);
      while (files[i-3].filehandle.read(buffer,512))
	 {
	   card.write(buffer,512);
	 }
      card.flush();
      //card.pubsync();
    }

  card.close();
}


